import os
import binascii
import codecs
from six.moves import input

HERE = os.path.abspath(os.path.dirname(__file__))


def render_template(template, destination, **kwargs):
    template = os.path.join(HERE, template)

    with codecs.open(template, 'r', encoding='utf-8') as f:
        raw_template = f.read()
        rendered = raw_template.format(**kwargs)
        with codecs.open(destination, 'w+', encoding='utf-8') as output:
            output.write(rendered)


def init(config_file):
    values = {}
    values['secret'] = binascii.b2a_hex(os.urandom(32))

    backend = input("Which backend to use? "
                    "(1 - postgresql, 2 - redis, default - memory) ").strip()

    if backend == '1':
        # Postgresql configuration
        postgresql_url = "postgres://postgres:postgres@localhost/postgres"
        values['storage_backend'] = "cliquet.storage.postgresql"
        values['storage_url'] = postgresql_url
        values['cache_backend'] = "cliquet.cache.postgresql"
        values['cache_url'] = postgresql_url
        values['permission_backend'] = "cliquet.permission.postgresql"
        values['permission_url'] = postgresql_url

    elif backend == '2':
        # Redis configuration
        values['storage_backend'] = "cliquet.storage.redis"
        values['storage_url'] = "redis://localhos:6379/1"
        values['cache_backend'] = "cliquet.cache.redis"
        values['cache_url'] = "redis://localhos:6379/2"
        values['permission_backend'] = "cliquet.permission.redis"
        values['permission_url'] = "redis://localhos:6379/3"

    else:
        # Memory configuration / default backend
        values['storage_backend'] = "cliquet.storage.memory"
        values['storage_url'] = ""
        values['cache_backend'] = "cliquet.cache.memory"
        values['cache_url'] = ""
        values['permission_backend'] = "cliquet.permission.memory"
        values['permission_url'] = ""

    render_template("kinto.tpl", config_file,
                    secret=values['secret'],
                    storage_backend=values['storage_backend'],
                    storage_url=values['storage_url'],
                    cache_backend=values['cache_backend'],
                    cache_url=values['cache_url'],
                    permission_backend=values['permission_backend'],
                    permission_url=values['permission_url'])



from __future__ import print_function
import argparse
import os
import sys
from cliquet.scripts import cliquet
from pyramid.scripts import pserve
from pyramid.paster import bootstrap

from kinto.config import init

CONFIG_FILE = 'config/kinto.ini'


def main(args=None):
    """The main routine."""
    if args is None:
        args = sys.argv[1:]

    parser = argparse.ArgumentParser(description="Kinto commands")
    parser.add_argument('--ini',
                        help='Application configuration file',
                        dest='ini_file',
                        required=False,
                        default=CONFIG_FILE)

    subparsers = parser.add_subparsers(title='subcommands',
                                       description='valid subcommands',
                                       help='init/start/migrate')

    parser_init = subparsers.add_parser('init')
    parser_init.set_defaults(which='init')

    parser_migrate = subparsers.add_parser('migrate')
    parser_migrate.set_defaults(which='migrate')

    parser_start = subparsers.add_parser('start')
    parser_start.set_defaults(which='start')

    args = vars(parser.parse_args())
    config_file = args['ini_file']

    if args['which'] == 'init':
        if not os.path.exists(config_file):
            init(config_file)
        else:
            print("%s already exist." % config_file, file=sys.stderr)
            sys.exit(1)

    elif args['which'] == 'migrate':
        env = bootstrap(config_file)
        cliquet.init_schema(env)

    elif args['which'] == 'start':
        pserve_argv = ['pserve', config_file, '--reload']
        pserve.main(pserve_argv)


if __name__ == "__main__":
    main()
